#========================================================================================
# package      DE_EPAGES::EPagesJ::t::SummaryLogger
# state        private
#========================================================================================
package DE_EPAGES::EPagesJ::t::SummaryLogger;
use base Test::Class;

use strict;

use DE_EPAGES::Core::API::File qw ( GetFileContent RunOnDirectory );
use DE_EPAGES::Core::API::Object::BaseObject;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::EPagesJ::API::Object::SummaryLogger;
use HTTP::Request;
use HTTP::Request::Common;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use Test::MockObject;
use Test::More;
use File::Basename;

sub testSummaryLogger : Test(no_plan) {
    my $self      = shift;
    my $LogString = 'Test';
    my $Logger    = DE_EPAGES::EPagesJ::API::Object::SummaryLogger->new( 'isLogging' => 1, );
    ## no critic (ProtectPrivateVars ProhibitNoWarnings)
    no warnings 'redefine';
    local *DE_EPAGES::EPagesJ::API::Object::CommunicationLogger::logger = sub { return $self->_mockLogger('4 bytes'); };
    $Logger->logCommunication($LogString);
}

sub testWithBinary : Test(no_plan) {
    my $self     = shift;
    my $Response = HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef,
        GetFileContent( 'fileContent1.pdf', { binary => 1 } ) );

    my $Logger = DE_EPAGES::EPagesJ::API::Object::SummaryLogger->new( 'isLogging' => 1, );

    no warnings 'redefine'; ## no critic (ProtectPrivateVars ProhibitNoWarnings)
    local *DE_EPAGES::EPagesJ::API::Object::CommunicationLogger::logger
    = sub { return $self->_mockLogger('8070 bytes'); };
    $Logger->logCommunication($Response);

}

sub testRequest : Test(no_plan) {
    my $self     = shift;
    my $Request = HTTP::Request->new(GET => 'http://www.example.com/');

    my $Logger = DE_EPAGES::EPagesJ::API::Object::SummaryLogger->new( 'isLogging' => 1, );

    no warnings 'redefine'; ## no critic (ProtectPrivateVars ProhibitNoWarnings)
    local *DE_EPAGES::EPagesJ::API::Object::CommunicationLogger::logger
    = sub { return $self->_mockLogger('GET http://www.example.com/'); };
    $Logger->logCommunication($Request);
}

sub assertLogContents {
    my ( $Content, $ExpectedString, $level ) = @_;
    like( $Content, qr/$ExpectedString/, $level . ' logging ' . $Content );
}

sub _mockLogger {
    my $self                 = shift;
    my ($assertedLogMessage) = @_;
    my $mock                 = Test::MockObject->new();
    my $assertedLogString    = $assertedLogMessage;
    my %mockMethods          = (
        'info' => sub { shift; assertLogContents( shift, $assertedLogString, 'info' ) },
        'is_info' => sub {1},
    );
    while ( my ( $method, $handler ) = each %mockMethods ) {
        $mock->mock( $method, $handler );
    }

    return $mock;
}

####### Main ############
RunScript(
    Debug => 1,
    Sub   => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    Store => GetTestSite(),
                    Sub   => sub {
                        Test::Class->runtests;
                    }
                );
            }
        );
    }
);
