use strict;
use warnings;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::EPagesJ::API::UriTemplateFactory qw ( CreateUriTemplates );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject );
use Test::More;

sub test {
    my $System = LoadRootObject();
    my $Site = $System->path('Test/UriTemplateSite');
    my $Uri = CreateUriTemplates($Site, ['{dsName}/{shopId}/someTest'])->{'{dsName}/{shopId}/someTest'}->uri->as_string;
    cmp_ok(index ($Uri, $Site->guid), '>', 0, $Uri . ' contains '.$Site->guid);
    cmp_ok(index ($Uri, GetTestSite()), '>', 0, $Uri . ' contains '. GetTestSite());
    cmp_ok(index ($Uri, $System->get('EPagesJBaseURL')), '==', 0, $Uri . ' contains '.$System->get('EPagesJBaseURL'));
}

sub init {
    LoadRootObject()->folder('Test')->folder('UriTemplateSite', {'Class' => LoadClassByAlias('Site')});
}

sub cleanup {
    LoadRootObject()->folder('Test')->delete;
}

####### Main ############
sub Main {
        RunOnStore(
            'Store' => GetTestSite(),
            'Sub'   => sub {
                init();
                test();
                cleanup();
                done_testing();
            }
        );
}

RunScript( Sub => \&Main, Debug => 1 );
