package DE_EPAGES::OAuthTokenClient::t::OAuthTokenClient;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use JSON qw( encode_json );
use Test::More tests => 10;
use Test::MockObject;

use strict;

## no critic qw(ProhibitLongLines)

sub test {
    my $Client = DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient->create({
        'BaseURL' => 'http://localhost:8088/rs-test/'
    });
    $Client->{'HttpClient'} = _mockHTTPClient( {
        'post'   => \&_assertCreateTokenContent,
        'delete' => \&_assertDeleteTokenContent,
        'get'    => \&_assertGetTokenContent,
    } );

    # check urls
    like($Client->_tokensUrl('MeinShop')->as_string, qr#shops/MeinShop/tokens$#, 'create token path ok'); ## no critic qw(ProhibitPrivateSubroutineUse)
    like($Client->_tokensUrl('Mein Shop'), qr#shops/Mein%20Shop/tokens$#, 'single token path (urlencoded) ok'); ## no critic qw(ProhibitPrivateSubroutineUse)
    #  create token
    my $Token = $Client->createToken('MeinShop', {
        'consumerKey'  => 'TestConsumer',
        'roles'        => [ 'sf', 'admin' ],
    });
    is($Token->token, '48ea81c1ea71475589a52711be81294b', 'Token is set to: '. $Token->token);
    is($Token->secret, 'b670fd5948274f7aaa690c9e212cb62e', 'Token secret is set to: '. $Token->secret);
    # delete token
    $Client->deleteToken('MeinShop', 'SomeToken');
    pass('deleteToken successful');
    # get token
    $Token = $Client->getToken('MeinShop', 'SomeToken');
    is($Token->token, '48ea81c1ea71475589a52711be81294b', 'getToken.token');
    is($Token->consumerKey, 'ba6f86bd0e4c434eb60929414180ec6c', 'getToken.consumerKey');
}

sub _mockHTTPClient {
    my ($hMockMethods) = @_;
    my $mockRemote     = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _assertGetTokenContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED), undef,
        '{"token":"48ea81c1ea71475589a52711be81294b","secret":"b670fd5948274f7aaa690c9e212cb62e","consumerKey":"ba6f86bd0e4c434eb60929414180ec6c","principal":{"name":"123"},"roles":["admin","sf"],"attributes":{}}'
    );
}

sub _assertCreateTokenContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/shops/MeinShop/tokens', 'post path for create' );
    is(
        $Content,
        encode_json( {
                'consumerKey'  => 'TestConsumer',
                'roles'        => [ 'sf', 'admin' ],
            }
        ),
        'post contents'
    );
    return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED), undef,
        '{"token":"48ea81c1ea71475589a52711be81294b","secret":"b670fd5948274f7aaa690c9e212cb62e","consumerKey":"ba6f86bd0e4c434eb60929414180ec6c","principal":{"name":"123"},"roles":["admin","sf"],"attributes":{}}'
    );
}

sub _assertDeleteTokenContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/shops/MeinShop/tokens/SomeToken', 'post path for delete' );
    return HTTP::Response->new( HTTP_NO_CONTENT, status_message(HTTP_NO_CONTENT), undef, undef );
}

__PACKAGE__->runScript;
