package DE_EPAGES::Product::t::ProductSlideshowResource;
use base DE_EPAGES::Core::API::Test;

use strict;
use Test::Exception;
use Test::More tests => 3;

use Data::Dumper;
use DE_EPAGES::Core::API::Object::BaseObject;
use DE_EPAGES::Product::API::Resource::ProductSlideshow;
use DE_EPAGES::Test::Mock::EpagesObject;

sub test {
    my $self = shift;

    $self->test_get;
    $self->test_get_noslideshow;
    $self->test_get_product_not_visible;
}

sub test_get {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductSlideshow->new;
    my @Images = (
        {
           Thumbnail => 'tumbnail-image.png',
           Small => 'small-image.png',
           HotDeal => 'hotdeal-image.png',
           MediumSmall => 'medium-small-image.png',
           Medium => 'medium-image.png',
           MediumLarge => 'medium-large-image.png',
           Large => 'large-image.png'
        },
        {
           Thumbnail => 'tumbnail-image.png',
           Small => undef,
           HotDeal => undef,
           MediumSmall => 'medium-small-image.png',
           Medium => 'medium-image.png',
           MediumLarge => 'medium-large-image.png',
           Large => 'large-image.png'
        },
    );
    my @ImagesSlideShow = map { { Sizes => $_ } } @Images;
    my $Product = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product', {
            Site => _MockShop(),
            IsVisible => 1,
            UseSlideshow => 1,
            ImagesSlideShow => \@ImagesSlideShow,
            WebPath => '/ProductWebPath',
        }
    );

    my $hExpected = {
        items => [
            map { { sizes => _ImageSizes($_, 'https://meinshop.de/ProductWebPath') } }
            @Images
        ]
    };
    is_deeply $Resource->get($Product), $hExpected, 'get slideshow images';
}

sub test_get_noslideshow {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductSlideshow->new;
    my $Product = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product', {
            Site => _MockShop(),
            IsVisible => 1,
            UseSlideshow => 0,
            SlideShowImages => [
                {
                    Thumbnail => 'tumbnail-image.png',
                    Small => undef,
                    HotDeal => undef,
                    MediumSmall => 'medium-small-image.png',
                    Medium => 'medium-image.png',
                    MediumLarge => 'medium-large-image.png',
                    Large => 'large-image.png'
                },
            ],
            GUID => '24AB755E-C277-4B43-9731-DBADEC61889D'
        }
    );

    throws_ok { $Resource->get( $Product ) } qr/ResourceNotFound/, 'get slideshow from product that has no slideshow';
}

sub test_get_product_not_visible {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductSlideshow->new;
    my $Product = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product', {
            Site => _MockShop(),
            IsVisible => 0,
            UseSlideshow => 1,
            SlideShowImages => [
                {
                    Thumbnail => 'tumbnail-image.png',
                    Small => undef,
                    HotDeal => undef,
                    MediumSmall => 'medium-small-image.png',
                    Medium => 'medium-image.png',
                    MediumLarge => 'medium-large-image.png',
                    Large => 'large-image.png'
                },
            ],
            GUID => '24AB755E-C277-4B43-9731-DBADEC61889D'
        }
    );

    throws_ok { $Resource->get( $Product ) } qr/ResourceNotFound/, 'get slideshow from product that is not visible';
}

sub _ImageSizes {
    my ($hImageSizes, $ImageBaseURL) = @_;

    my $ToAbsolute = sub {
        local $_ = $_[0];
        return defined $_ ? "$ImageBaseURL/$_" : undef;
    };

    return [
        map { _Image( $_, $ToAbsolute->($hImageSizes->{$_}) ) }
        keys %$hImageSizes
    ];
}

sub _Image {
    my ($Classifier, $URL) = @_;

    return {
        classifier => $Classifier,
        url => $URL,
    };
}

sub _MockShop {
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', {
            ProtocolAndServerSSL => 'https://meinshop.de',
        }
    );
}

__PACKAGE__->runScript;
