#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::RemoteIndexClient
# state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::RemoteIndexClient;
use base Test::Class;

use strict;
use warnings;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::EPagesJ::API::Object::HTTPClient;
use DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use JSON;
use Test::MockObject;
use Test::More;
use URI::URL ();

sub testIndexShop : Test(no_plan) {
    my $self   = shift;
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
         $self->_fakeUrls(),
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertCreateIndexContents } )
    );
    my $Task = $Client->indexShop(1);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testDropIndex : Test(no_plan) {
    my $self   = shift;
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        $self->_fakeUrls(),
        'HttpClient' => $self->_mockHTTPClient( { 'delete' => \&_assertDropIndexContents } )
    );
    my $Task = $Client->dropIndex(2);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testIndexItems : Test(no_plan) {
    my $self   = shift;
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        $self->_fakeUrls(),
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertIndexItemsContents } )
    );
    my $Task = $Client->indexItems([map {{'resourceId' => $_ }} qw (100 101 102)], 3);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testDeleteItems : Test(no_plan) {
    my $self   = shift;
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        $self->_fakeUrls(),
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertDeleteItemsContents } )
    );
    my $Task = $Client->deleteItems([map {{'resourceId' => $_ }} qw (100)], 4);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testGetTask : Test(no_plan) {
    my $self   = shift;
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        $self->_fakeUrls(),
        'HttpClient' => $self->_mockHTTPClient( { 'get' => \&_assertgetTaskContents } )
    );
    my $Task = $Client->getTask('http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563');
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testIndexShops : Test(no_plan) {
    my $self   = shift;
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        $self->_fakeUrls(),
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertIndexShopsContents } )
    );
    my $Task = $Client->indexShops([map {{'resourceId' => $_ }} qw (100 101 102)], 3);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub _mockHTTPClient {
    my $self           = shift;
    my ($hMockMethods) = @_;
    my $mockRemote     = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _fakeUrls {
    my $self = shift;
    return (
    'IndexShopUrl'     => URI::URL->new('http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4'),
    'IndexShopsUrl'    => URI::URL->new('http://localhost:8088/rs-test/product/Store'),
    'IndexItemsUrl'    => URI::URL->new('http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/update'),
    'DeleteItemsUrl'   => URI::URL->new('http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/delete'),
    );
}

sub _assertCreateIndexContents {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4', 'post path for CreateIndex' );
    is(
        $Content,
        encode_json( {
                'replicationToken' => 1,
            }
        ),
        'post contents'
    );
    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertDropIndexContents {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4', 'delete url for DropIndex' );
    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertgetTaskContents {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563', 'url for task' );
    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}
sub _assertIndexShopsContents {
    shift;
    my ($URL, $hHeader, $Content) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store', 'post url for indexShops');
    is($Content, encode_json({
    'replicationToken' => 3,
    'indexItems' => {
        items => [
            { 'resourceId' => '100' },
            { 'resourceId' => '101' },
            { 'resourceId' => '102' },
        ],
    }
    }), 'update items contents.');

    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertIndexItemsContents {
    shift;
    my ($URL, $hHeader, $Content) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/update', 'post url for InsertOrUpdateItems');
    is($Content, encode_json({
    'replicationToken' => 3,
    'indexItems' => {
        items => [
            { 'resourceId' => '100' },
            { 'resourceId' => '101' },
            { 'resourceId' => '102' },
        ],
    }
    }), 'update items contents.');

    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertDeleteItemsContents {
    shift;
    my ($URL, $hHeader, $Content) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/delete', 'post url for DeleteItems');
    is($Content, encode_json({
    'replicationToken' => 4,
    'indexItems' => {
        'items' => [{ 'resourceId' => '100' }],
    }
    }), 'delete items contents.');

    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );

}

####### Main ############
RunScript(
    Sub => sub {
        Test::Class->runtests;
      }
);
